package service

import (
	"fmt"

	omd "git.loafle.net/overflow/model/discovery"
	omm "git.loafle.net/overflow/model/meta"
	omu "git.loafle.net/overflow/model/util"
	osm "git.loafle.net/overflow/service_matcher-go"
	ouej "git.loafle.net/overflow/util-go/encoding/json"
	"git.loafle.net/overflow_scanner/probe/discovery/session"
	"git.loafle.net/overflow_scanner/probe/internal/matcher"
)

func scanUDP(discoverySession session.DiscoverySession, targetPort *omd.Port) error {
	portNumber, err := ouej.NumberToInt(targetPort.PortNumber)
	if err != nil {
		return fmt.Errorf("Service scan port[%s] error %v ", targetPort.PortNumber, err)
	}

	ms := matcher.GetUDPMatchers()
	mi := osm.NewMatchInfo(targetPort.Host.Address, portNumber)

	for i := 0; i < len(ms); i++ {
		m := ms[i]
		p := osm.NewPacket(targetPort.UDPLayer.LayerPayload(), len(targetPort.UDPLayer.LayerPayload()))

		if err := m.Match(mi, 0, p); err == nil {
			s := &omd.Service{
				Key:            m.Key(),
				Port:           targetPort,
				MetaCryptoType: omm.ToMetaCryptoType(omm.MetaCryptoTypeEnumNONE),
				DiscoveredDate: omu.NowPtr(),
				Metadata:       m.Meta(),
				Name:           m.Name(),
			}

			discoverySession.AddService(s)

			return nil
		}
	}

	return nil
}
